"""
Building module. Has all the stuff you need to build you hotel. Coolness!
"""
#------------------------------IMPORTS-----------------------------------------#

import types

import pygame
from pygame.locals import *
from pygame.colordict import THECOLORS as COLORS

import data

#------------------------------GLOBALS-----------------------------------------#

TT_COLOR = 0
TT_IMAGE = 1

ATTRIB_NO = 0
ATTRIB_BASE = 1
ATTRIB_UPGRADE = 2


#------------------------------CLASSES-----------------------------------------#

class BuildingBlock:
    """
    A base building block that can be changed into anything. Something like a
    foundation block for building stuff on it.
    """
    def __init__(self, cost=0, running_cost=0, rect=None, texture_representation=None, attributes=ATTRIB_NO):
        """
        """
        try:
            self.rect = rect
            self.size = (1, 1)
            self.cost = int(cost)
            self.running_cost = int(running_cost)
            self.buyable_block = False
            self.allow_base = False
            self.allow_upgrade = False
            
            self.has_something = False
            self.rent = 0
            self.bought = False
            
            self.occupying_block = None
            
            self.base_brick = False
            self.upgrade_brick = False

            if type(texture_representation) in types.StringTypes:
                self.texture_type = TT_IMAGE
                self.sur_block, sur_rect = data.load_image(texture_representation)
            
            self.set_attrib(attributes)                
        except Exception , error:
            error = "Initializing the BuildingBlock class failed:\n %s" % (error)
            raise
    
    def set_attrib(self, attributes):
        """
        Sets the attributes of the class.
        """
        try:
            self.attributes = attributes
            if ATTRIB_BASE == attributes & ATTRIB_BASE:
                self.base_brick = True
            if ATTRIB_UPGRADE == attributes & ATTRIB_UPGRADE:
                self.upgrade_brick = True
                
        except Exception , error:
            error = "set_attrib in the BuildingBlock class failed:\n %s" % (error)
            raise
        
    def can_be_placed(self, tiles, (x, y)):
        """
        Return True is the brick be set in that specific tile grid in that
        location, False otherwise.
        """
        return True

class ReceptionBlock(BuildingBlock):
    """
    A reception tile. It can only be placed at the first line.
    """
    def __init__(self, *args, **kargs):
        BuildingBlock.__init__(self, *args, **kargs)
        self.size = (1, 2)
        self.base_brick = False
        self.upgrade_brick = True
        self.buyable_block = False
        self.rent = 0
        self.running_cost = -1
        self.occupying_block = None
        self.has_something = False
        
    def can_be_placed(self, tiles, (x, y)):
        """
        The reception can only be placed at the first level.
        """
        return not x

class RoomBlock(BuildingBlock):
    """
    A room block. It's buyable!
    """
    def __init__(self, occupied_image=None, *args, **kargs):
        BuildingBlock.__init__(self, *args, **kargs)
        self.sur_empty = self.sur_block
        
        self.occupied_image = occupied_image
        if None != occupied_image:
            self.sur_occupied, rect = data.load_image(data.filepath(occupied_image))
        
        self.occupying_block = None
        self.base_brick = False
        self.upgrade_brick = True
        self.buyable_block = True
        self.rent = 0.5
        self.bought = False
        self.has_something = False
        
    def can_be_placed(self, tiles, (x, y)):
        """
        The room can be placed on any level that's not first.
        """
        return x
    
    def occupy(self):
        """
        Occupy the apartment, thus switching it's image.
        """
        self.sur_block, rect = data.load_image(data.filepath(self.occupied_image))
        self.bought = True
        
#------------------------------FUNCTIONS---------------------------------------#

