"""
A game logic module to implement some people...
"""
#------------------------------IMPORTS-----------------------------------------#

import os
import sys
import copy

import pygame
from pygame.locals import *
from pygame.colordict import THECOLORS as COLORS

import game_blocks

#------------------------------GLOBALS-----------------------------------------#


#------------------------------CLASSES-----------------------------------------#

class GameLogic:
    """
    The logic class that will house the pplz that buy stuff :).
    """
    def __init__(self, num_of_people, tiles):
        try:
            self.tiles = tiles
            self.num_of_people = num_of_people
            
            self.free_people = self.num_of_people
            self.paying_people = 0
            self.popularity_percent = 1.0
            
        except Exception , error:
            error = "Initializing the GameLogic class failed:\n %s" % (error)
            raise
    
    def buy(self, new_percent, screen, tiles=None):
        """
        """
        try:
            if not None == tiles:
                self.tiles = tiles
            self.popularity_percent = new_percent
            people_buying = self.tiles.buyable_tiles * self.popularity_percent
            print "%s people are buying stuff! (buyable = %s)" % (people_buying, self.tiles.buyable_tiles)
            self.tiles.buyable_tiles -= people_buying
            self.paying_people -= people_buying
            monthly_income = 0.0
            for i in xrange(int(people_buying)):
                monthly_income += self.tiles.buy(people_buying, screen)
            return monthly_income
            
        except Exception , error:
            error = "Initializing the GameLogic class failed:\n %s" % (error)
            raise


class BrickInfo:
    """
    Base brick info.
    """
    def __init__(self, image, cost, running_cost=0, occupied_image=None):
        """
        """
        self.image = image
        self.cost = int(cost)
        self.occupied_image = occupied_image
        self.running_cost = float(running_cost)

class TileGrid:
    """
    The tiles that represeent places the player can build on. Some hard coded
    values ahead!
    """
    def __init__(self, (x, y), base_tile_size):
        """
        Initialize the tile / layering system.
        """
        try:
            self.occupied_tiles = 0
            self.buyable_tiles = 0
            self.num_of_horizontal_tiles = x / base_tile_size
            self.num_of_vertical_tiles = (y / base_tile_size)
            self.total_tiles = self.num_of_horizontal_tiles * self.num_of_vertical_tiles
            print "Initial grid size = (%s, %s) = %s" % (self.num_of_vertical_tiles, self.num_of_horizontal_tiles, self.total_tiles)
            
            temp_list = []
            self.tiles = []
            for i in xrange(self.num_of_vertical_tiles):
                for j in xrange(self.num_of_horizontal_tiles):
                    if (0 == i) and (j > 4):
                        #If this is the first layer and not on the sea we can
                        #build on it:
                        buildable = True
                    else:
                        buildable = False
                    cur_rect = Rect(j * base_tile_size, ((self.num_of_vertical_tiles - i) * base_tile_size - base_tile_size), base_tile_size, base_tile_size)
                    empty_brick = game_blocks.BuildingBlock(rect=cur_rect)
                    empty_brick.allow_base = buildable
                    empty_brick.occupying_block = None
                    temp_list.append(empty_brick)
                    del empty_brick
                    
                self.tiles.append(temp_list)
                temp_list = []

        except Exception , error:
            error = "Initializing the tile_grid class failed:\n %s" % (error)
            raise
    
    def place(self, block, (x, y)):
        """
        Place the given block in the start tile [y][x] (may be more is the
        block's size is more than (1, 1).
        """
        try:
            blit_rect = None
            allow_blit = True
            backup_tiles = copy.deepcopy(self.tiles)
            base_tile = self.tiles[y][x]
            
            if (base_tile.allow_base) and (block.base_brick) and (block.can_be_placed(self.tiles, (y, x))):
                for i in xrange(block.size[0]):
                    for j in xrange(block.size[1]):
                        if (y + i < len(self.tiles)) and (x + j < len(self.tiles[y+i])):
                            current_tile = self.tiles[y + i][x + j]
                            
                            if (self.num_of_vertical_tiles > y + 1) and (not self.tiles[y + 1][x].has_something):
                                self.tiles[y + 1][x].allow_base = True
                                if (self.num_of_horizontal_tiles > x + j + 1) and (not self.tiles[y + 1][x + 1].has_something):
                                    self.tiles[y + 1][x + 1].allow_base = True
                                if (0 < x) and (not self.tiles[y + 1][x - 1].has_something):
                                    self.tiles[y + 1][x - 1].allow_base = True
                            
                            if current_tile.allow_base:
                                current_tile.base_brick = False
                                current_tile.allow_base = False
                                current_tile.allow_upgrade = True
                                current_tile.occupying_block = copy.deepcopy(block)
                                current_tile.has_something = True
                            else:
                                allow_blit = False
                                    
                            if None == blit_rect:
                                blit_rect = current_tile.rect
                            else:
                                blit_rect.union_ip(current_tile.rect)
                        else:
                            allow_blit = False
                            
            elif (base_tile.allow_upgrade) and (block.upgrade_brick) and (block.can_be_placed(self.tiles, (y, x))):
                for i in xrange(block.size[0]):
                    for j in xrange(block.size[1]):
                        if (y + i < len(self.tiles)) and (x + j < len(self.tiles[y+i])):
                            current_tile = self.tiles[y + i][x + j]
                            if current_tile.allow_upgrade:
                                current_tile.allow_upgrade = False
                                current_tile.occupying_block = copy.deepcopy(block)
                                current_tile.has_something = True
                            else:
                                allow_blit = False
                            
                            if None == blit_rect:
                                blit_rect = current_tile.rect
                            else:
                                blit_rect.union_ip(current_tile.rect)
                        else:
                            allow_blit = False
            else:
                allow_blit = False

            if allow_blit:
                if (block.buyable_block) and (None != backup_tiles[y][x].occupying_block):
                    self.buyable_tiles += 1
                return blit_rect
            else:
                self.tiles = backup_tiles
                return None
            
        except Exception , error:
            error = "place in the tile_grid class failed:\n %s" % (error)
            raise
    
    def buy(self, houses_to_buy, screen):
        """
        Buy the number of houses specified.
        """
        try:
            for i in xrange(self.num_of_vertical_tiles):
                for j in xrange(self.num_of_horizontal_tiles):
                    if (None != self.tiles[i][j].occupying_block):
                        if (self.tiles[i][j].occupying_block.buyable_block):
                            if (not self.tiles[i][j].occupying_block.bought):
                                self.tiles[i][j].occupying_block.occupy()
                                screen.blit(self.tiles[i][j].occupying_block.sur_block, self.tiles[i][j].rect)
                                return self.tiles[i][j].occupying_block.rent
            return 0
            
        except Exception , error:
            error = "buy in the tile_grid class failed:\n %s" % (error)
            raise


#------------------------------FUNCTIONS---------------------------------------#
