"""#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#
#                               PyIcy Tower                                    #
#                             ---------------                                  #
#                                                                              #
#   Author:                                                                    #
#           Michael Sverdlin                                                   #
#   Date:                                                                      #
#           03.07                                                              #
#   Purpose:                                                                   #
#           An Icy tower clone, using python. Warmup for PyWeek4.              #
#                                                                              # 
#   TODO:                                                                      #
#                                                                              #
#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#"""
#------------------------------IMPORTS-----------------------------------------#

import os
import sys
import copy
import time
import threading
import ConfigParser

import pygame
from pygame.locals import *
from pygame.colordict import THECOLORS as COLORS
pygame.init()                               #Initialize all of the pygame parts

import data     #Having data in a folder in the parent dir SUCKS IMHO...
import menu
import intro
import game_blocks
from game_logic import *

#------------------------------GLOBALS-----------------------------------------#

DEBUG = True
MAIN_INI_FILE = "game.ini"
INI_TILES_SECTION = "tiles"
INI_INTRO_SECTION = "intro"
INI_BRICK_SECTION = "bricks"
INI_INGAME_SECTION = "ingame"
INI_GENERAL_SECTION = "general"
INI_HUD_SPLICES_SECTION = "hud_areas"

EVENT_BUY = USEREVENT + 2
CLEAR_ICON = USEREVENT + 7
global g_dirty_rects

#------------------------------CLASSES-----------------------------------------#
        
class Game:
    """
    This is an instance of the game. Classer rock too much not to use them for
    for this.
    
    Members:
    Exported Methods:
        run - Starts the game.
    """
    
    def __init__(self, game_ini_path, screen):
        """
        Initialize the game class.
        """
        try:
            self.screen = screen
            self.brick = None
            self.brick_cache = {}
            #Parse the game's attributes:
            self.game_ini_path = game_ini_path
            assert os.path.isfile(self.game_ini_path), "The given ini file (%s) doesn't exist!"
            self._parse_ini_file()
            
            self.money = float(self.ingame_options["starting_money"])
            self.running_cost = 0

            self.cur_block_rect = None
            
            self.blocks_bought = 0
            self.blocks_downed = 0
            self.spent = 0
            self.income = 0
            
            try:
                self.font_obj = pygame.font.Font(data.filepath(self.screen_options["font"]), 30)
            except IOError:
                self.font_obj = pygame.font.SysFont(pygame.font.get_default_font(), 30)
            
            self.rect_icon = ini_list_to_rect(self.hud_areas_options["status_area"])
            self.sur_icon = pygame.Surface((self.rect_icon.width, self.rect_icon.height))
            self.sur_icon.fill((253, 253, 103, 255))
                        
        except Exception , error:
            error = "Initializing the game class failed:\n %s" % (error)
            raise
    
    def _parse_ini_file(self):
        """
        Parse the game's ini file for all the important attributes:
        screen_width, screen_height, screen_flags, background_image
        """
        try:
            #Open the given ini file and read it:
            ini_parser = ConfigParser.ConfigParser()
            ini_parser.read(self.game_ini_path)
            #Read the screen options:
            main_options = ini_parser.items(INI_GENERAL_SECTION)
            self.screen_options = dict(main_options)
            #Read the ingame options:
            self.ingame_options = dict(ini_parser.items(INI_INGAME_SECTION))
            #Read hud areas:
            self.hud_areas_options = dict(ini_parser.items(INI_HUD_SPLICES_SECTION))
            #Read the tile stuff:
            self.tile_options = dict(ini_parser.items(INI_TILES_SECTION))
            
            #Parse the bricks info:
            self.brick_image_to_price = dict(ini_parser.items(INI_BRICK_SECTION))
            for key in self.brick_image_to_price.keys():
                encoded_info = self.brick_image_to_price[key].split(" ", 4)
                self.brick_image_to_price[key] = BrickInfo(encoded_info[0], encoded_info[1], encoded_info[2], encoded_info[3])
            
        except Exception , error:
            error = "_parse_ini_file in the game class failed:\n %s" % (error)
            raise        
        
    def run(self):
        """
        Runs the game, whatever that means.
        """
        try:
            global g_dirty_rects
            g_dirty_rects = []
            
            #Initialize the screen and background:
            bg, rect = data.load_image(data.filepath(self.screen_options["bg_image"]))
            self.screen.blit(bg, rect)
            
            pygame.display.flip()
            self._show_start_menu()
        except Exception , error:
            error = "run in the game class failed:\n %s" % (error)
            raise
    
    def _show_start_menu(self):
        """
        Show the start menu with all the game options.
        """
        try:
            test_item = menu.MenuItem("Start", 120, 376, COLORS["green"], self.main_game_loop, font=self.screen_options["font"])
            test_item2 = menu.MenuItem("Quit", 120, 445, COLORS["green"], self.end, font=self.screen_options["font"])
            start_menu = menu.StartMenu(self.screen, [test_item, test_item2])
            start_menu.start_menu()
            
        except Exception , error:
            error = "_show_start_menu in the game class failed:\n %s" % (error)
            raise
        
    def end(self):
        """
        A callback that does nothing, so that you can quit the game.
        """
        pass
    
    def options(self):
        print "No options :("
        self._show_start_menu()
        
    def time_to_buy(self):
        """
        """
        self.screen.blit(self.hud.sur_hud, self.hud.sur_rect)
        self.screen.blit(self.sur_icon, self.rect_icon)
        
        self.income += self.game_logic.buy(1.0, self.screen, self.tile_grid)
        self.income_surface = self.font_obj.render(str(self.income) + "K", True, COLORS["white"], (110, 254, 4, 255))
        self.screen.blit(self.income_surface, self.income_rect)
        
        self.money += self.income
        self.money_surface = self.font_obj.render(str(self.money) + "K", True, COLORS["white"], (252, 79, 14, 255))
        self.screen.blit(self.money_surface, self.money_rect)
        
        pygame.display.flip()
        
    def main_game_loop(self):
        """
        This will feature the main game loop.
        """
        try:
            global g_dirty_rects
            g_dirty_rects = []
            
            if pygame.cursors.arrow != pygame.mouse.get_cursor():
                pygame.mouse.set_cursor(*pygame.cursors.arrow)
            
            
            #Start a timer for people buying:
            pygame.time.set_timer(EVENT_BUY, 1000 * int(self.screen_options["month"]))
            
            self.clock = pygame.time.Clock()
            
            #Load the ingame background:
            self.ingame_bg_sur, self.ingame_bg_rect = data.load_image(data.filepath(self.ingame_options["bg_image"]))
            self.screen.blit(self.ingame_bg_sur, self.ingame_bg_rect)
            
            self.hud = menu.HUD(ini_list_to_rect(self.hud_areas_options["hud_rect"]), self.game_ini_path, help_rect=self.rect_icon, idle_help=self.sur_icon)
            self.hud.add_part(menu.HUDPart(ini_list_to_rect(self.hud_areas_options["gray_bb_area"]), self.gray_build_brick, self.hud_areas_options["base_help"]))
            self.hud.add_part(menu.HUDPart(ini_list_to_rect(self.hud_areas_options["yellow_bb_area"]), self.recept_brick, self.hud_areas_options["reception_help"]))
            self.hud.add_part(menu.HUDPart(ini_list_to_rect(self.hud_areas_options["room_bb_area"]), self.small_room_brick, self.hud_areas_options["room_help"]))
            self.screen.blit(self.hud.sur_hud, self.hud.sur_rect)
            

            self.money_rect = ini_list_to_rect(self.hud_areas_options["money_area"])
            self.money_surface = self.font_obj.render(str(self.money) + "K", True, COLORS["white"])
            self.screen.blit(self.money_surface, self.money_rect)
            
            self.income_rect = ini_list_to_rect(self.hud_areas_options["income_area"])
            self.income_surface = self.font_obj.render(str(self.income)  + "K", True, COLORS["white"])
            self.screen.blit(self.income_surface, self.income_rect)
            
            pygame.display.flip()
            
            #Create the tiles:
            self.tile_grid = TileGrid((int(self.screen_options["screen_width"]) - 100, int(self.screen_options["screen_height"])), int(self.tile_options["tile_size"]))
            self.game_logic = GameLogic(100, self.tile_grid)
            self.is_mouse_down = False
            self.last_pos = (0, 0)
            
            self.should_run = True
            while self.should_run:
                
                for event in pygame.event.get():
                    #Check if we need to quit:
                    if QUIT == event.type:
                        self.should_run = False
                    #Quit if alt+f4 was hit:
                    elif ((KEYDOWN == event.type) and (K_F4 == event.key) and (KMOD_LALT == event.mod & KMOD_LALT)):
                        pygame.event.post(pygame.event.Event(QUIT))
                    
                    elif MOUSEBUTTONDOWN == event.type:
                        self.is_mouse_down = True
                        
                    elif MOUSEBUTTONUP == event.type:
                        self.is_mouse_down = False
                    
                    elif MOUSEMOTION == event.type:
                        self.last_pos = event.pos
                    
                    elif CLEAR_ICON == event.type:
                        pygame.time.set_timer(CLEAR_ICON, 0)
                        self.sur_icon = pygame.Surface((self.rect_icon.width, self.rect_icon.height))
                        self.sur_icon.fill((253, 253, 103, 255))
                        self.screen.blit(self.sur_icon, self.rect_icon)
                        
                    elif (EVENT_BUY == event.type):
                        print "Fps:", self.clock.get_fps()
                        self.time_to_buy()                    
                    
                    #Send the event to the hud:
                    self.hud.handle_input(event, screen=self.screen)
                    g_dirty_rects.append(self.hud.help_rect)
                    
                    if (self.is_mouse_down) and (None != self.brick) and (680 > self.last_pos[0]):
                        current_tile_x = self.last_pos[0] / int(self.tile_options["tile_size"])
                        current_tile_y = (int(self.screen_options["screen_height"]) - 1 - self.last_pos[1]) / int(self.tile_options["tile_size"])
                        #print "**", current_tile_x, current_tile_y
                        if (self.money >= self.brick.cost):
                            blit_rect = self.tile_grid.place(self.brick, (current_tile_x, current_tile_y))
        
                            if None != blit_rect:
                                self.money -= self.brick.cost
                                self.income += self.brick.running_cost
                                self.income_surface = self.font_obj.render(str(self.income) + "K", True, COLORS["white"], (110, 254, 4, 255))
                                self.money_surface = self.font_obj.render(str(self.money) + "K", True, COLORS["white"], (252, 79, 14, 255))
                                self.screen.blit(self.hud.sur_hud, self.hud.sur_rect)
                                self.screen.blit(self.sur_icon, self.rect_icon)
                                self.screen.blit(self.money_surface, self.money_rect)
                                self.screen.blit(self.income_surface, self.income_rect)
                                g_dirty_rects.append(self.money_rect)
                                g_dirty_rects.append(self.income_rect)
                                self.spent += self.brick.cost
                                self.blocks_bought += 1
                                self.screen.blit(self.brick.sur_block, blit_rect)
                                g_dirty_rects.append(blit_rect)
                                
                        else:
                            self._show_no_money()
                        
                pygame.display.update(g_dirty_rects)
                #pygame.display.flip()
                g_dirty_rects = []
                self.clock.tick(100)
                
        except Exception , error:
            pygame.time.set_timer(USEREVENT, 0)
            error = "_main_game_loop in the game class failed:\n %s" % (error)
            raise
        
    def _show_no_money(self):
        """
        Show the no money sign on the hud.
        """
        try:
            self.sur_icon, rect_no_money = data.load_image(data.filepath(self.hud_areas_options["hud_no_money"]))
            self.sur_icon.set_colorkey((254, 254, 254, 255))
            self.screen.blit(self.sur_icon, self.rect_icon)
            g_dirty_rects.append(self.rect_icon)
            pygame.time.set_timer(CLEAR_ICON, int(self.ingame_options["icon_time"]) * 1000)
        except Exception , error:
            error = "_show_no_money in the game class failed:\n %s" % (error)
            raise            
        
    def brick_factory(self, image="reg_gray_brick.tif", size=(1, 1), attributes=game_blocks.ATTRIB_NO):
        """
        A factory for bricks...
        """
        self.brick = game_blocks.BuildingBlock(attributes = attributes, texture_representation = data.filepath(os.path.join(self.screen_options["brick_folder"], image)))
        self.brick.size = size
        
    def gray_build_brick(self):
        """
        Sets the current brick as a gray building brick.
        """
        cur_tile_info = self.brick_image_to_price["base"]
        image = data.filepath(os.path.join(self.screen_options["brick_folder"], cur_tile_info.image))
        self.brick = game_blocks.BuildingBlock(cur_tile_info.cost, running_cost = cur_tile_info.running_cost, attributes = game_blocks.ATTRIB_BASE, texture_representation = image)
        self.brick.size = (1, 1)
        
    def recept_brick(self):
        """
        Sets the current brick as a reception brick.
        """
        cur_tile_info = self.brick_image_to_price["reception"]
        image = data.filepath(os.path.join(self.screen_options["brick_folder"], cur_tile_info.image))
        self.brick = game_blocks.ReceptionBlock(cur_tile_info.cost, running_cost = cur_tile_info.running_cost, attributes = game_blocks.ATTRIB_BASE, texture_representation = image)

    def small_room_brick(self):
        """
        Sets the current brick as a room brick.
        """
        cur_tile_info = self.brick_image_to_price["small_room"]
        image = data.filepath(os.path.join(self.screen_options["brick_folder"], cur_tile_info.image))
        occupied_image = data.filepath(os.path.join(self.screen_options["brick_folder"], cur_tile_info.occupied_image))
        self.brick = game_blocks.RoomBlock(occupied_image=occupied_image, cost=cur_tile_info.cost, running_cost = cur_tile_info.running_cost, attributes = game_blocks.ATTRIB_BASE, texture_representation = image)

#------------------------------FUNCTIONS---------------------------------------#

def check_point((x, y), rect):
    """
    Checks if the point represented by (x,y) is in the area of the rect.
    """
    if (x > rect.left) and (x < rect.right):
        if (y < rect.bottom) and (y > rect.top):
            return True
        
    return False

def ini_list_to_rect(ini_area_string):
    """
    Tries to create a rect from the ini_area_string given.
    The string should be something like that: "x y w h".
    """
    split_measures = ini_area_string.split(" ", 4)
    split_measures = [int(i) for i in split_measures]
    
    if 4 == len(split_measures):
        return Rect(split_measures[0], split_measures[1], split_measures[2], split_measures[3])
    raise ValueError("Rect area (%s) in wrong format, stop changing ini :)" % ini_area_string)

#------------------------------MAIN--------------------------------------------#

def main():
    try:
        #Remove the python CL arg, we don't need it...
        if "python" in sys.argv[0]:
            sys.argv = sys.argv[1:]
            
        ini_parser = ConfigParser.ConfigParser()
        ini_parser.read(data.filepath(MAIN_INI_FILE))
        
        #Read the screen options:
        main_options = dict(ini_parser.items(INI_GENERAL_SECTION))
        intro_options = dict(ini_parser.items(INI_INTRO_SECTION))
        
        screen = pygame.display.set_mode((int(main_options["screen_width"]), int(main_options["screen_height"])))
        pygame.display.set_caption("PyTower - Where you too can build stuff")
        
        del ini_parser
        
        intro_instance = intro.Intro(screen)
        #Set hot spots for facr intro:
        def intro_image(image_name):
            return data.filepath(os.path.join(main_options["intro_folder"], image_name))
        
        #Create the face hot spots:
        hot_spots = []
        hot_spots.append(intro.HotSpot(intro_image("dollar sign.jpg"), ini_list_to_rect(intro_options["dollar_hs"]), (100, 100)))
        hot_spots.append(intro.HotSpot(intro_image("euro.jpg"), ini_list_to_rect(intro_options["euro_hs"]), (543, 159)))
        hot_spots.append(intro.HotSpot(intro_image("gold_teeh.jpg"), ini_list_to_rect(intro_options["gold_teeth_hs"]), (60, 329)))
        hot_spots[2].sur_hs.set_colorkey(COLORS["black"])
        
        #sub = intro.Subtitle("MooLalaMoo", intro_image(intro_options["some_font"]), words_in_sec = 3, start_pos=(1, 420))
        
        quit = False
        skip = False

        intro_instance.set_scene(intro_image("face.jpg"), hot_spots)
        
        answer = intro_instance.start_loop()
        if intro.LE_QUIT == answer:
            quit = True
        if intro.LE_SKIP == answer:
            skip = True

        hot_spots = []
        hot_spots.append(intro.HotSpot(intro_image("cow_threat.jpg"), ini_list_to_rect(intro_options["cow_hs"]), (483, 100)))
        
        
        if (not quit):
            intro_instance.set_scene(intro_image("SunChainedRaft.png"), hot_spots, move = True)
            answer = intro_instance.start_loop()
            if intro.LE_QUIT == answer:
                quit = True        
       
        #Start the game already!
        if not quit:
            game_instance = Game(data.filepath(MAIN_INI_FILE), screen)
            game_instance.run()
        
    except Exception, error:
        print "Program failed, here's a detailed explanation:"
        print error
        if DEBUG:
            print "Taceback stack:"
            import traceback
            print traceback.print_exc()